@@ -79,7 +79,7 @@ gwpr_A <- function(bw, data, SDF, ID_list, formula, p, longlat, adaptive,
     weight <- GWmodel::gw.weight(as.numeric(subsample$dist), bw=bw_of_total, kernel=kernel, adaptive=adaptive)
     subsample$wgt <- as.vector(weight)
     Psubsample <- plm::pdata.frame(subsample, index = index, drop.index = FALSE, row.names = FALSE,
-                                   stringsAsFactors = default.stringsAsFactors())
+                                   stringsAsFactors = FALSE)
     plm_subsample <- plm::plm(formula=formula, model=model, data=Psubsample,
                               effect = effect, index=index, weights = wgt,
                               random.method = random.method)
